/*
 * Program        : readFP.c
 *
 * Objectives     : To read the fp-tree generated by FP-growth function
 *
 * Assumptions    : Item ID in dataset file should start from zero.
 *                  I.e. if there are NUM items in the dataset, 
 *                  the item ID should be from 0 to NUM-1.
 *
 * Variations     : 
 *
 * Platform       : SunOS/Linux
 * Compiler       : gcc
 *
 * Programmer     : Raymond Chi-Wing Wong
 * Creation Date  : Oct, 2002
 * Last Update    : Jun 9, 2004
 *
 * Reference      :
 *   1. J. Han, J. Pei, and Y. Yin, 
 *      "Mining Frequent Patterns without Candidate Generation", 
 *      Proc. ACM SIGMOD 2000.
 *   2. R. C.-W. Wong, A. W.-C. Fu and K. Wang
 *      "MPIS: Maximal-Profit Item Selection with Cross-Selling Considerations"
 *      The 2003 IEEE International Conference on Data Mining (ICDM)
 *      
 */

#include "readFP.h"



/*
 * Function  : readFP
 * Purpose   : to read the freq. itemsets outputed by FP-growth
 * Parameters: 
 *   1. infile: the file name storing the freq. itemsets outputed by FP-growth
 *   2. upperLimit: the maximum size of the itemsets mined
 */
largeitemPtr *readFP(STRING infile, int *upperLimit)
{
	largeitemPtr *returnValue;

	FILE *filePtr;

	char *tempBuffer;


	int i, j, k;

	filePtr = (FILE *) fopen(infile, "r");
	if (filePtr == NULL)
	{
		printf("File %s does not exist!\n", infile);
		exit(1);
	}

	tempBuffer = (char *) malloc(sizeof(char)*MAX_LEN);

	fscanf(filePtr, "K_{max} = %d\n\n", upperLimit);

	returnValue = (largeitemPtr *) malloc(sizeof(largeitem)*(*upperLimit));

	for (i = 0; i < (*upperLimit); i++)
	{
		returnValue[i] = (largeitemPtr) malloc(sizeof(largeitem));

		fgets(tempBuffer, MAX_LEN, filePtr);
		if (strcmp(tempBuffer, "") != 0)
		{
			sscanf(tempBuffer, "%d", &(returnValue[i]->noOfItemSet));
		}
		else
		{
			for (j = i; j < (*upperLimit); j++)
			{
				returnValue[j]->noOfItemSet = 0;
			}
			break;
		}

		returnValue[i]->item = (itemsetPtr *) malloc(sizeof(itemsetPtr)*(returnValue[i]->noOfItemSet));

		for (j = 0; j < returnValue[i]->noOfItemSet; j++)
		{
			returnValue[i]->item[j] = (itemsetPtr) malloc(sizeof(itemset));

			returnValue[i]->item[j]->itemNo = (int *) malloc(sizeof(int)*(i+1));

			for (k = 0; k < i+1; k++)
			{
				fscanf(filePtr, "%d ", &(returnValue[i]->item[j]->itemNo[k]));
			}
			fscanf(filePtr, "[%d]\n", &(returnValue[i]->item[j]->freq));
		}

		//fgets(tempBuffer, MAX_LEN, filePtr);
	}
	
	


	fclose(filePtr);

	return returnValue;
}
